@@ -11,7 +11,7 @@ use Business::OnlinePayment::HTTPS;
 use vars qw($VERSION $DEBUG @ISA $me);
 
 @ISA = qw(Business::OnlinePayment::HTTPS);
-$VERSION = '0.09';
+$VERSION = '0.12';
 $VERSION = eval $VERSION; # modperlstyle: convert the string into a number
 
 $DEBUG = 0;
@@ -217,6 +217,7 @@ sub submit {
   }
   if ($self->test_transaction()) {
     $content{'login'} = 'TESTTERMINAL';
+    $self->server('testgtwy.ippay.com') if $self->server eq 'gtwy.ippay.com';
   }
   $self->content(%content);
 
@@ -253,6 +254,10 @@ sub submit {
     $amount =~ s/\.//;
   }
 
+  my $name = length($content{name})
+               ? $content{name}
+               : $content{first_name}. ' '. $content{last_name};
+
   my $check_number = $content{check_number} || "100"  # make one up
     if($content{account_number});
 
@@ -269,6 +274,13 @@ sub submit {
     unless $country;
   $country = uc($country) if $country;
 
+  my $ship_name =
+    length($content{ship_name})
+      ? $content{ship_name}
+      : (length($content{ship_first_name}) || length($content{ship_last_name}))
+        ? $content{ship_first_name}. ' '. $content{ship_last_name}
+        : '';
+
   my $ship_country =
     country2code( $content{ship_country}, LOCALE_CODE_ALPHA_3 );
   $ship_country  = country_code2code( $content{ship_country},
@@ -320,7 +332,7 @@ sub submit {
     $self->revmap_fields(
                           CustomerPO          => 'CustomerPO',
                           ShippingMethod      => 'ShippingMethod',
-                          ShippingName        => 'ship_name',
+                          ShippingName        => \$ship_name,
                           ShippingAddr        => \%shippingaddr,
                         );
 
@@ -347,7 +359,7 @@ sub submit {
                           Track1              => 'track1',
                           Track2              => 'track2',
                           ACH                 => \%ach,
-                          CardName            => 'name',
+                          CardName            => \$name,
                           DispositionType     => 'DispositionType',
                           TotalAmount         => \$amount,
                           FeeAmount           => 'FeeAmount',
@@ -359,7 +371,7 @@ sub submit {
                           BillingCountry      => \$country,
                           BillingPhone        => 'phone',
                           Email               => 'email',
-                          UserIPAddr          => 'customer_ip',
+                          UserIPAddress       => 'customer_ip',
                           UserHost            => 'UserHost',
                           UDField1            => 'UDField1',
                           UDField2            => 'UDField2',
@@ -377,11 +389,11 @@ sub submit {
                                 ENCODING    => 'us-ascii',
                               );
   $writer->xmlDecl();
-  $writer->startTag('JetPay');
+  $writer->startTag('ippay');
   foreach ( keys ( %req ) ) {
     $self->_xmlwrite($writer, $_, $req{$_});
   }
-  $writer->endTag('JetPay');
+  $writer->endTag('ippay');
   $writer->end();
 
   warn "$post_data\n" if $DEBUG > 1;
@@ -448,7 +460,8 @@ sub _xmlwrite {
   if ( $item eq 'ACH' ) {
     $att{'Type'} = $self->{_content}->{'account_type'}
       if $self->{_content}->{'account_type'}; #necessary so we don't pass empty?
-    $att{'SEC'}  = 'PPD';
+    $att{'SEC'}  = $self->{_content}->{'nacha_sec_code'}
+                 || ( $att{'Type'} =~ /business/i ? 'CCD' : 'PPD' );
   }
 
   $writer->startTag($item, %att);
@@ -588,7 +601,7 @@ from content(%content):
       BillingCountry      => 'country',           # forced to ISO-3166-alpha-3
       BillingPhone        => 'phone',
       Email               => 'email',
-      UserIPAddr          => 'customer_ip',
+      UserIPAddress        => 'customer_ip',
       UserHost            => 'UserHost',
       UDField1            => 'UDField1',
       UDField2            => 'UDField2',
@@ -625,6 +638,15 @@ Current maintainer: Ivan Kohler <ivan-ippay@freeside.biz>
 
 Reverse Authorization patch from dougforpres
 
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (c) 1999 Jason Kohles
+Copyright (c) 2002-2003 Ivan Kohler
+Copyright (c) 2008-2022 Freeside Internet Services, Inc.
+
+All rights reserved. This program is free software; you can redistribute it
+and/or modify it under the same terms as Perl itself.
+
 =head1 ADVERTISEMENT
 
 Need a complete, open-source back-office and customer self-service solution?
